﻿##################################################
# GetEvent1221Info.ps1
# Used to consolidate 1221 events from mailbox servers.
# 
# Created: 10/26/2006
# Author: Tyson Kopczynski
##################################################
param([int] $Days, [string] $ExportFile)

##################################################
# Functions
##################################################
#-------------------------------------------------
# Get-RemoteEventLog
#-------------------------------------------------
# Usage:        Used to collect remote EventLog information from a machine.
# $Machine:     The name of the machine. ("MyServer")   
# $Log:         The name of the EventLog. ("Application")   

function Get-RemoteEventLog{
    param ($Machine, $Log)

    trap{Continue}

    new-object System.Diagnostics.Eventlog $Log, $Machine
    }

#-------------------------------------------------
# Export-DataTable
#-------------------------------------------------
# Usage:        Used to export a DataSet to a CSV file.
# $Data:        A DataSet object.
# $FileName:    The name of the export CSV file.

function Export-DataTable{
    param ($Data, $FileName)

    $Null = `
        [System.Reflection.Assembly]::LoadWithPartialName( `
            "System.Windows.Forms")
    
    trap{write-host "[ERROR] $_" -Foregroundcolor Red; Continue}

    if ($FileName -eq ""){
        $exFileName = new-object System.Windows.Forms.saveFileDialog
        $exFileName.DefaultExt = "csv"
        $exFileName.Filter = "CSV (Comma delimited)(*.csv)|*.csv"
		$exFileName.ShowHelp = $True
        $exFileName.ShowDialog()

        $FileName = $exFileName.FileName
        }

    if ($FileName -ne ""){
        $LogFile = new-object System.IO.StreamWriter($FileName, $False)

         for ($i=0; $i -le $Data.Columns.Count-1; $i++){
            $LogFile.Write($Data.Columns[$i].ColumnName)
            
            if ($i -lt $Data.Columns.Count-1){
                $LogFile.Write(",")
                }
            }
        
        $LogFile.WriteLine()

        foreach ($Row in $Data.Rows){
            for ($i=0; $i -le $Data.Columns.Count-1; $i++){
                $LogFile.Write($Row[$i].ToString())
                
                if ($i -lt $Data.Columns.Count-1){
                    $LogFile.Write(",")
                    }
                }
                
            $LogFile.WriteLine()
            }
        
        $LogFile.Close()
        }
    }

##################################################
# Main
##################################################
#--------------------
# Load Libraries
#--------------------
. .\LibraryGen.ps1

#--------------------
# Set Config Vars
#--------------------
$ScriptName = "GetEvent1221Info.ps1"
$ScriptUsage = "Used to consolidate 1221 events from mailbox servers."
$ScriptCommand = "$ScriptName -Days value -ExportFile value"
$ScriptParams = "Days = The number of days to filter events by.", `
    "ExportFile = The export CSV file path/filename."
$ScriptExamples = "$ScriptName 5 ""report.csv"""
$ErrorLogName = "GetEvent1221Info.log"
$Date = Date

#--------------------
# Verify Required Parameters
#--------------------
if ($args[0] -match '-(\?|(h|(help)))'){
    write-host
    Get-ScriptHeader $ScriptName $ScriptUsage
    Show-ScriptUsage $ScriptCommand $ScriptParams $ScriptExamples
    Return
    }

if (!$Days){
    write-host
    write-host "Please specify the number of days!" -Foregroundcolor Red
    write-host
    Get-ScriptHeader $ScriptName $ScriptUsage
    Show-ScriptUsage $ScriptCommand $ScriptParams $ScriptExamples
    Return
    }

#--------------------
# Define DataSets
#--------------------
$ServersTable = new-object System.Data.DataTable
$ServersTable.TableName = "Servers"
[Void]$ServersTable.Columns.Add("Name")
[Void]$ServersTable.Columns.Add("Status")

$EventsTable = new-object System.Data.DataTable
$EventsTable.TableName = "Servers"
[Void]$EventsTable.Columns.Add("Server")
[Void]$EventsTable.Columns.Add("TimeWritten",[DateTime])
[Void]$EventsTable.Columns.Add("Database")
[Void]$EventsTable.Columns.Add("MB")

#--------------------
# Begin Script
#--------------------
# Setup ErrorLog
$ScriptName + " Ran on: " + $Date | out-file $ErrorLogName

write-host
Get-ScriptHeader $ScriptName $ScriptUsage
write-host

#--------------------
# Get Servers and Status
#--------------------
write-host "Getting Mailbox Servers" -NoNewLine
$MailboxServers = get-mailboxserver
write-host `t "[DONE]" -Foregroundcolor Green

write-host "Getting Status Info" -NoNewLine

$MailboxServers | foreach-object -Begin {$i=0;} `
    -Process {&{$Ping = new-object Net.NetworkInformation.Ping;
            $MBServerName = $_.Name;
            trap{"Ping [ERROR]: " + $MBServerName + " $_" | out-file `
                $ErrorLogName -Append; Continue};
            $Result = $Ping.Send($MBServerName);
            if ($Result.Status -eq "Success"){  `
                [Void]$ServersTable.Rows.Add($MBServerName, "Online")} `
            else{[Void]$ServersTable.Rows.Add($MBServerName, "Offline")};
            $i = $i+1;
            write-progress -Activity "Pinging Servers - $($MBServerName)" `
                -Status "Progress:" `
                -PercentComplete ($i / $MailboxServers.Count * 100)}}

write-host `t "[DONE]" -Foregroundcolor Green

# Write status info to ErrorLog
$ServersTable | out-file $ErrorLogName -Append

#--------------------
# Get Event Info
#--------------------
write-host "Getting Event Info"  -NoNewLine

$OnlineServers = $ServersTable.Select("Status = 'Online'")

foreach ($Server in $OnlineServers){
    &{
        trap{"Event Info [Error]: " + $Server.Name + " $_" | `
            out-file $ErrorLogName -Append; Continue}

        $Events = Get-RemoteEventLog $Server.Name "Application"
        
        # This may take a long time depending on the number of servers
        write-progress -Activity "Querying Events From - $($Server.Name)" `
            -Status "This may take sometime..." 
        
        $1221Events = $Events.Entries | where {$_.EventID -eq "1221" -and `
            $_.TimeWritten -ge $Date.AddDays(-$Days)}

        foreach ($1221Event in $1221Events){
            &{
                $Message = $1221Event | select Message
                $TimeWritten = $1221Event.get_timewritten()
         
                # This RegEx strips out the database name from the message
                $Database = [Regex]::Match($Message, '"[^"\r\n]*"')
                $Database = $Database.Value.Replace('"', "")
                
                # This RegEx strips out size of the whitespace
                $MB = [Regex]::Match($Message, '[0-9]+')
            
                [Void]$EventsTable.Rows.Add($Server.Name, $TimeWritten, `
                    $Database, $MB)
            }
            }
        
        write-progress -Activity "Querying Events From - $($Server.Name)" `
            -Status "Done" -completed $True
    }   
    }

write-host `t "[DONE]" -Foregroundcolor Green

#--------------------
# Export Data to CSV File
#--------------------
$Null = Export-DataTable $EventsTable $ExportFile

write-host
write-host "Script is now DONE!" -Foregroundcolor Green
write-host "Check the $ErrorLogName for errors." -Foregroundcolor Yellow 
# SIG # Begin signature block
# MIIMywYJKoZIhvcNAQcCoIIMvDCCDLgCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU6f/QoNv92uIAm+XqecN91sfE
# XhqgggorMIIDJzCCApCgAwIBAgIBATANBgkqhkiG9w0BAQQFADCBzjELMAkGA1UE
# BhMCWkExFTATBgNVBAgTDFdlc3Rlcm4gQ2FwZTESMBAGA1UEBxMJQ2FwZSBUb3du
# MR0wGwYDVQQKExRUaGF3dGUgQ29uc3VsdGluZyBjYzEoMCYGA1UECxMfQ2VydGlm
# aWNhdGlvbiBTZXJ2aWNlcyBEaXZpc2lvbjEhMB8GA1UEAxMYVGhhd3RlIFByZW1p
# dW0gU2VydmVyIENBMSgwJgYJKoZIhvcNAQkBFhlwcmVtaXVtLXNlcnZlckB0aGF3
# dGUuY29tMB4XDTk2MDgwMTAwMDAwMFoXDTIwMTIzMTIzNTk1OVowgc4xCzAJBgNV
# BAYTAlpBMRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93
# bjEdMBsGA1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRp
# ZmljYXRpb24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVt
# aXVtIFNlcnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhh
# d3RlLmNvbTCBnzANBgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEA0jY2aovXwlue2oFB
# Yo847kkEVdbQ7xwblRZH7xhINTpS9CtqBo87L+pW46+GjZ4X9560ZXUCTe/LCaIh
# Udib0GfQug2SBhRz1JPLlyoAnFxODLz6FVL88kRu2hFKbgifLy3j+ao6hnO2RlNY
# yIkFvYMRuHM/qgeN9EJN50CdHDcCAwEAAaMTMBEwDwYDVR0TAQH/BAUwAwEB/zAN
# BgkqhkiG9w0BAQQFAAOBgQAmSCwWwlj66BZ0DKqqX1Q/8tfJeGBeXm43YyJ3Nn6y
# F8Q0ufUIhfzJATj/Tb7yFkJD57taRvvBxhEf8UqwKEbJw8RCfbz6q1lu1bdRiBHj
# pIUZa4JMpAwSremkrj/xw0llmozFyD4lt5SZu5IycQfwhl7tUCemDaYj+bvLpgcU
# QjCCA04wggK3oAMCAQICAQowDQYJKoZIhvcNAQEFBQAwgc4xCzAJBgNVBAYTAlpB
# MRUwEwYDVQQIEwxXZXN0ZXJuIENhcGUxEjAQBgNVBAcTCUNhcGUgVG93bjEdMBsG
# A1UEChMUVGhhd3RlIENvbnN1bHRpbmcgY2MxKDAmBgNVBAsTH0NlcnRpZmljYXRp
# b24gU2VydmljZXMgRGl2aXNpb24xITAfBgNVBAMTGFRoYXd0ZSBQcmVtaXVtIFNl
# cnZlciBDQTEoMCYGCSqGSIb3DQEJARYZcHJlbWl1bS1zZXJ2ZXJAdGhhd3RlLmNv
# bTAeFw0wMzA4MDYwMDAwMDBaFw0xMzA4MDUyMzU5NTlaMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCB
# iQKBgQDGuLknYK8L45FpZdt+je2R5qrxvtXt/m3ULH/RcHf7JplXtN0/MLjcIepo
# jYGS/C5LkTWEIPLaSrq0/ObaiPIgxSGSCUeVoAkcpnm+sUwd/PGKblTSaaHxTJM6
# Qf591GR7Y0X3YGAdMR2k6dMPi/tuJiSzqP/l5ZDUtMLcUGCuWQIDAQABo4GzMIGw
# MBIGA1UdEwEB/wQIMAYBAf8CAQAwQAYDVR0fBDkwNzA1oDOgMYYvaHR0cDovL2Ny
# bC50aGF3dGUuY29tL1RoYXd0ZVByZW1pdW1TZXJ2ZXJDQS5jcmwwHQYDVR0lBBYw
# FAYIKwYBBQUHAwIGCCsGAQUFBwMDMA4GA1UdDwEB/wQEAwIBBjApBgNVHREEIjAg
# pB4wHDEaMBgGA1UEAxMRUHJpdmF0ZUxhYmVsMi0xNDQwDQYJKoZIhvcNAQEFBQAD
# gYEAdrKc7hOfG/YtNJKURXM03I5rLlz8TH2J68No8deZDy4dF8i1Fou+zYoFBvIZ
# SToDWwXJII5tUuF2gaDDZYoiZ+QcU1M3Rr+81y/re57QFEVsQCEI4l11dmYwHvTf
# goovvfOiDL8d258UoppyN02wd0joSj8JzlUZLO/mByThr+wwggOqMIIDE6ADAgEC
# AhAaB4sUlpgPs6Hq/y+uOpfhMA0GCSqGSIb3DQEBBQUAMFUxCzAJBgNVBAYTAlpB
# MSUwIwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQD
# ExZUaGF3dGUgQ29kZSBTaWduaW5nIENBMB4XDTA2MTAwNTAwMDAwMFoXDTA4MTAw
# NDIzNTk1OVowczELMAkGA1UEBhMCVVMxEzARBgNVBAgTCkNhbGlmb3JuaWExEDAO
# BgNVBAcTB09ha2xhbmQxFzAVBgNVBAoTDmNvbXBhbnlhYmMuY29tMQswCQYDVQQL
# EwJJVDEXMBUGA1UEAxMOY29tcGFueWFiYy5jb20wggEiMA0GCSqGSIb3DQEBAQUA
# A4IBDwAwggEKAoIBAQDa7qCoFx5g8HE6wIc3IEYVU2xbeWFL57b/zPkRwPZcvJfY
# ypC/IijwGYV0fOZOUmiMXL+/EoMlMszmGYGiQbeweDVijGYYfAyDeiXsa3NDt1ud
# dbN3iBjoYST/fetkdL1e6yactF29MpHMtVwA5Mgr6TXTr4msMBAdG487J/ujKLAb
# Bv8SvsV8olNqeOAkqcUJG2xcNEU6z/wrEhfriPgHHsGyDf4gKztCUeroWGyVJPz5
# Rq3Wpgi07zs5VKxNiSufjOh8ew5gboVylJ2efvHhLUb62YoW4Vq2nQGmjL3HX5w5
# wsEsCKoD8ZLGT8qlK1YMmfoK/VZJsP+ydfGHtKcJAgMBAAGjgdgwgdUwDAYDVR0T
# AQH/BAIwADA+BgNVHR8ENzA1MDOgMaAvhi1odHRwOi8vY3JsLnRoYXd0ZS5jb20v
# VGhhd3RlQ29kZVNpZ25pbmdDQS5jcmwwHwYDVR0lBBgwFgYIKwYBBQUHAwMGCisG
# AQQBgjcCARYwHQYDVR0EBBYwFDAOMAwGCisGAQQBgjcCARYDAgeAMDIGCCsGAQUF
# BwEBBCYwJDAiBggrBgEFBQcwAYYWaHR0cDovL29jc3AudGhhd3RlLmNvbTARBglg
# hkgBhvhCAQEEBAMCBBAwDQYJKoZIhvcNAQEFBQADgYEAT0BOmxgOLTPPqPhCKqf9
# 17nHCn1w0Bp0Qg/MJnsMuAqRuIL+Wlvxrp6DAPfzvrTI1wKd5b5TwuKwub0u45CI
# qSdglyF4efza6ksSmEPRaUGEnY2/p8Nc9ku7K+MPBTnFGfPFwfvFschi+8KMP1f/
# c4+QHoGcSRgRxhJjb+T9YxwxggIKMIICBgIBATBpMFUxCzAJBgNVBAYTAlpBMSUw
# IwYDVQQKExxUaGF3dGUgQ29uc3VsdGluZyAoUHR5KSBMdGQuMR8wHQYDVQQDExZU
# aGF3dGUgQ29kZSBTaWduaW5nIENBAhAaB4sUlpgPs6Hq/y+uOpfhMAkGBSsOAwIa
# BQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEWMCMGCSqGSIb3
# DQEJBDEWBBRb1JiKvKT1ay6t5vx7cwoxnCHa7jANBgkqhkiG9w0BAQEFAASCAQA+
# DLyc7D/d3M1wR67P7XEFoF+JhI4OxrGWsPpStOesX4gQ2W7AcB1RCovVdzQCccN2
# k8nwJI5W5oB/9FmtGXTJIy/epK5vp+yw14q0AzupFpSJ67c2UIGX0EogD4XHCQ5j
# bvVZEQQa3d72YqLqkZcVg9gymHa37Tb09pdY64mO8S+oiEDISASLmLflv0bLZpLG
# GBk7HVzgDNH5INrQPNFPcIma+oiIVqn/9hfgCQb8sDTtWl+mrzEbsKJrrNkSQWBh
# /VOPPYtuYutqyKCNO7Cpg4/hjnGwY+CxOGCrbjxSDDPxDnYPT2FynlqwD8K55Dnl
# oKMMm+Hrncer/Hgo297t
# SIG # End signature block
